#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <pthread.h>
#include <linux/types.h> 
#include <gtk/gtk.h>
#include <linux/videodev.h>
#include "gqcam.h"
#include "frontend.h"
#include "save.h"

GtkWidget *drawing_area;
static GdkPixmap *pixmap = NULL;

void setbrightness(GtkAdjustment *adj, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_pic.brightness = (adj->value)*256;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setwhite(GtkAdjustment *adj, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_pic.whiteness = (adj->value)*256;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setcontrast(GtkAdjustment *adj, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_pic.contrast = (adj->value)*256;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void sethue(GtkAdjustment *adj, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_pic.hue = (adj->value)*256;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setcolor(GtkAdjustment *adj, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_pic.colour = (adj->value)*256;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setsizefull(GtkWidget *widget, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_win.width = 320;
  camera->vid_win.height = 240;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setsizehalf(GtkWidget *widget, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_win.width = 160;
  camera->vid_win.height = 120;
  camera->update_camera = 1;
  clearimg(camera);
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setsizequarter(GtkWidget *widget, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_win.width = 80;
  camera->vid_win.height = 60;
  camera->update_camera = 1;
  clearimg(camera);
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setdepth4(GtkWidget *widget, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_pic.depth = 4;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void setdepth6(GtkWidget *widget, struct Camera *camera)
{
  pthread_mutex_lock(&camera->pic_mutex);
  camera->vid_pic.depth = 6;
  camera->update_camera = 1;
  pthread_mutex_unlock(&camera->pic_mutex);
}

void clearimg(struct Camera *camera)
{
  GdkRectangle update_rec;

  configure_event(GTK_WIDGET (camera->drawing_area),NULL, camera);

  update_rec.x = 0;
  update_rec.y = 0;
  update_rec.width = 320;
  update_rec.height = 240;
  gtk_widget_draw (camera->drawing_area, &update_rec);
  return;
}

static gint 
configure_event (GtkWidget *widget, GdkEventConfigure *event, struct Camera *camera)
{
  if (camera->pixmap)
    gdk_pixmap_unref(camera->pixmap);

  camera->pixmap = gdk_pixmap_new(widget->window,
				  widget->allocation.width,
				  widget->allocation.height,
				  -1);
  gdk_draw_rectangle (camera->pixmap,
		      widget->style->white_gc,
		      TRUE,
		      0, 0,
		      widget->allocation.width,
		      widget->allocation.height);
  return TRUE;
}

static gint 
expose_event (GtkWidget *widget, GdkEventExpose *event, struct Camera *camera)
{
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		  camera->pixmap,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);
  return FALSE;
}

void freeze(GtkWidget *widget, struct Camera *camera)
{
  if (GTK_TOGGLE_BUTTON (widget)->active)
    {
      gdk_threads_enter();
      //pthread_mutex_lock(&camera->pic_mutex);
      camera->frozen=1;
    } else {
      gdk_threads_leave();
      //pthread_mutex_unlock(&camera->pic_mutex);
      camera->frozen=0;
    } 
}

int create_frontend(struct Camera *camera)
{
  GtkWidget *window;
  GtkWidget *topbox;
  GtkWidget *outerbox;
  GtkWidget *innerbox;
  GtkWidget *scalebox;
  GtkWidget *frame;
  GtkObject *adjb;
  GtkWidget *scaleb;
  GtkObject *adjw;
  GtkWidget *scalew;
  GtkObject *adjc;
  GtkWidget *scalec;
  GtkWidget *radiobutton;
  GSList *radiogroup;
  GtkWidget *button;
  GdkDrawable *drawable;
  guchar *gbuff;
  GdkColor c;
  char title[50] = "Gqcam - ";
  struct video_clip vid_clips[32];
  int ib=130, ic=30, iw=150, i, count=32;

  strcat(title, camera->vid_caps.name);
/*
  for (i = 0; i < count; i++) {
    vid_clips[i].x      = 0;
    vid_clips[i].y      = 0;
    vid_clips[i].width  = 0;
    vid_clips[i].height = 0;
  }
  
  vid_win.clips = vid_clips;
  vid_win.clipcount = 0;

  ib = (vid_pic.brightness)/256;
  ic = (vid_pic.contrast)/256;
  iw = (vid_pic.whiteness)/256;
*/  
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), title);

  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);
  
  gtk_container_set_border_width (GTK_CONTAINER (window), 2);

  /* Create topbox */
  topbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), topbox);

  /* Create the drawing area */
  frame = gtk_frame_new(NULL);
  gtk_box_pack_start (GTK_BOX(topbox), frame, TRUE, TRUE, 0);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gtk_widget_show(frame);

  camera->drawing_area = gtk_drawing_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (camera->drawing_area), 320, 240);
  gtk_container_add(GTK_CONTAINER(frame), camera->drawing_area);
  gtk_widget_show (camera->drawing_area);
/*
    gtk_signal_connect (GTK_OBJECT (camera->drawing_area), "expose_event",
		      (GtkSignalFunc) display, camera);
*/
/*------ bp -------*/

  gtk_signal_connect (GTK_OBJECT (camera->drawing_area), "expose_event",
		      (GtkSignalFunc) expose_event, camera);
  gtk_signal_connect (GTK_OBJECT(camera->drawing_area),"configure_event",
		      (GtkSignalFunc) configure_event, camera);

/*-----------------*/

  if (camera->greyscale)
    create_controls_greyscale(topbox, camera);
  else
    create_controls_color(topbox, camera);

  /* Create a couple buttons */
  innerbox = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start( GTK_BOX (topbox), innerbox, 1, 1, 0);
  gtk_widget_show (innerbox);
  
  button = gtk_button_new_with_label ("Snap Picture");
  gtk_box_pack_start( GTK_BOX (innerbox), button, 1, 1, 0);
  gtk_widget_show(button);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (save_dialog), camera);

  button = gtk_toggle_button_new_with_label ("Freeze Picture");
  gtk_box_pack_start(GTK_BOX (innerbox), button, 1, 1, 0);
  gtk_widget_show(button);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) freeze, camera);

  button = gtk_button_new_with_label ("Quit");
  gtk_box_pack_start( GTK_BOX (innerbox), button, 1, 1, 0);
  gtk_widget_show(button);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
	 	      GTK_SIGNAL_FUNC (delete_event), NULL);

  
  gtk_widget_show (topbox);
  gtk_widget_show (window);

  return 0;
}



